@@ -94,6 +94,17 @@ int callback_openid_userinfo_valid_json (const struct _u_request * request, stru
   return U_CALLBACK_CONTINUE;
 }
 
+int callback_openid_userinfo_valid_json_dpop (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  if (0 == o_strcmp(u_map_get(request->map_header, "Authorization"), "Bearer " ACCESS_TOKEN) && u_map_get(request->map_header, I_HEADER_DPOP) != NULL) {
+    json_t * j_response = json_loads(userinfo_json, JSON_DECODE_ANY, NULL);
+    ulfius_set_json_body_response(response, 200, j_response);
+    json_decref(j_response);
+  } else {
+    response->status = 401;
+  }
+  return U_CALLBACK_CONTINUE;
+}
+
 int callback_openid_userinfo_valid_char (const struct _u_request * request, struct _u_response * response, void * user_data) {
   if (0 == o_strcmp(u_map_get(request->map_header, "Authorization"), "Bearer " ACCESS_TOKEN)) {
     ulfius_set_string_body_response(response, 200, userinfo_char);
@@ -194,13 +205,13 @@ START_TEST(test_iddawc_userinfo_invalid_parameters)
   ck_assert_int_eq(i_init_session(&i_session), I_OK);
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_ERROR_PARAM);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR_PARAM);
   i_clean_session(&i_session);
   
   ck_assert_int_eq(i_init_session(&i_session), I_OK);
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_ERROR_PARAM);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR_PARAM);
   i_clean_session(&i_session);
 }
 END_TEST
@@ -217,7 +228,7 @@ START_TEST(test_iddawc_userinfo_invalid_response)
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_ERROR_UNAUTHORIZED);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR_UNAUTHORIZED);
   i_clean_session(&i_session);
   
   ulfius_stop_framework(&instance);
@@ -237,7 +248,7 @@ START_TEST(test_iddawc_userinfo_response_unauthorized)
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_ACCESS_TOKEN, "error",
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_ERROR_UNAUTHORIZED);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR_UNAUTHORIZED);
   ck_assert_ptr_eq(i_get_str_parameter(&i_session, I_OPT_USERINFO), NULL);
   ck_assert_ptr_eq(i_session.j_userinfo, NULL);
   i_clean_session(&i_session);
@@ -259,7 +270,7 @@ START_TEST(test_iddawc_userinfo_response_empty)
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_ERROR);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR);
   ck_assert_ptr_eq(i_get_str_parameter(&i_session, I_OPT_USERINFO), NULL);
   ck_assert_ptr_eq(i_session.j_userinfo, NULL);
   i_clean_session(&i_session);
@@ -281,7 +292,7 @@ START_TEST(test_iddawc_userinfo_response_char)
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_OK);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_OK);
   ck_assert_str_eq(i_get_str_parameter(&i_session, I_OPT_USERINFO), userinfo_char);
   ck_assert_ptr_eq(i_session.j_userinfo, NULL);
   i_clean_session(&i_session);
@@ -304,7 +315,7 @@ START_TEST(test_iddawc_userinfo_response_json)
   ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
                                                     I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_OK);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_OK);
   ck_assert_int_eq(json_equal(i_session.j_userinfo, j_userinfo), 1);
   i_clean_session(&i_session);
   json_decref(j_userinfo);
@@ -314,6 +325,65 @@ START_TEST(test_iddawc_userinfo_response_json)
 }
 END_TEST
 
+START_TEST(test_iddawc_userinfo_response_json_dpop)
+{
+  struct _i_session i_session;
+  struct _u_instance instance;
+  json_t * j_userinfo = json_loads(userinfo_json, JSON_DECODE_ANY, NULL);
+  jwk_t * jwk;
+  ck_assert_int_eq(ulfius_init_instance(&instance, 8080, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", NULL, "/userinfo", 0, &callback_openid_userinfo_valid_json_dpop, NULL), U_OK);
+  ck_assert_int_eq(ulfius_start_framework(&instance), U_OK);
+  
+  ck_assert_int_eq(i_init_session(&i_session), I_OK);
+  ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
+                                                    I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
+                                                    I_OPT_USE_DPOP, 1,
+                                                    I_OPT_TOKEN_JTI_GENERATE, 16,
+                                                    I_OPT_DPOP_SIGN_ALG, "RS256",
+                                                    I_OPT_NONE), I_OK);
+  ck_assert_int_eq(r_jwk_init(&jwk), RHN_OK);
+  ck_assert_int_eq(r_jwk_import_from_json_str(jwk, jwk_privkey_rsa_str), RHN_OK);
+  ck_assert_int_eq(r_jwks_append_jwk(i_session.client_jwks, jwk), RHN_OK);
+  r_jwk_free(jwk);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_OK);
+  ck_assert_int_eq(json_equal(i_session.j_userinfo, j_userinfo), 1);
+  i_clean_session(&i_session);
+  json_decref(j_userinfo);
+  
+  ulfius_stop_framework(&instance);
+  ulfius_clean_instance(&instance);
+}
+END_TEST
+
+START_TEST(test_iddawc_userinfo_response_json_dpop_invalid)
+{
+  struct _i_session i_session;
+  struct _u_instance instance;
+  jwk_t * jwk;
+  ck_assert_int_eq(ulfius_init_instance(&instance, 8080, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", NULL, "/userinfo", 0, &callback_openid_userinfo_valid_json_dpop, NULL), U_OK);
+  ck_assert_int_eq(ulfius_start_framework(&instance), U_OK);
+  
+  ck_assert_int_eq(i_init_session(&i_session), I_OK);
+  ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
+                                                    I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
+                                                    I_OPT_USE_DPOP, 0,
+                                                    I_OPT_TOKEN_JTI_GENERATE, 16,
+                                                    I_OPT_DPOP_SIGN_ALG, "RS256",
+                                                    I_OPT_NONE), I_OK);
+  ck_assert_int_eq(r_jwk_init(&jwk), RHN_OK);
+  ck_assert_int_eq(r_jwk_import_from_json_str(jwk, jwk_privkey_rsa_str), RHN_OK);
+  ck_assert_int_eq(r_jwks_append_jwk(i_session.client_jwks, jwk), RHN_OK);
+  r_jwk_free(jwk);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 0), I_ERROR_UNAUTHORIZED);
+  i_clean_session(&i_session);
+  
+  ulfius_stop_framework(&instance);
+  ulfius_clean_instance(&instance);
+}
+END_TEST
+
 START_TEST(test_iddawc_userinfo_response_jwt_signed)
 {
   struct _i_session i_session;
@@ -332,7 +402,7 @@ START_TEST(test_iddawc_userinfo_response_jwt_signed)
                                                     I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
                                                     I_OPT_NONE), I_OK);
   ck_assert_int_eq(r_jwks_append_jwk(i_session.server_jwks, jwk_sign), RHN_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_OK);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 1), I_OK);
   ck_assert_int_eq(json_equal(i_session.j_userinfo, j_userinfo), 1);
   i_clean_session(&i_session);
   json_decref(j_userinfo);
@@ -343,35 +413,6 @@ START_TEST(test_iddawc_userinfo_response_jwt_signed)
 }
 END_TEST
 
-START_TEST(test_iddawc_userinfo_response_jwt_encrypted)
-{
-  struct _i_session i_session;
-  struct _u_instance instance;
-  jwk_t * jwk_decrypt;
-  json_t * j_userinfo = json_loads(userinfo_json, JSON_DECODE_ANY, NULL);
-  
-  ck_assert_int_eq(ulfius_init_instance(&instance, 8080, NULL, NULL), U_OK);
-  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", NULL, "/userinfo", 0, &callback_openid_userinfo_valid_jwt_encrypted, NULL), U_OK);
-  ck_assert_int_eq(ulfius_start_framework(&instance), U_OK);
-  ck_assert_int_eq(r_jwk_init(&jwk_decrypt), RHN_OK);
-  ck_assert_int_eq(r_jwk_import_from_json_str(jwk_decrypt, jwk_privkey_rsa_str), RHN_OK);
-  
-  ck_assert_int_eq(i_init_session(&i_session), I_OK);
-  ck_assert_int_eq(i_set_parameter_list(&i_session, I_OPT_USERINFO_ENDPOINT, "http://localhost:8080/userinfo",
-                                                    I_OPT_ACCESS_TOKEN, ACCESS_TOKEN,
-                                                    I_OPT_NONE), I_OK);
-  ck_assert_int_eq(r_jwks_append_jwk(i_session.client_jwks, jwk_decrypt), RHN_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_OK);
-  ck_assert_int_eq(json_equal(i_session.j_userinfo, j_userinfo), 1);
-  i_clean_session(&i_session);
-  json_decref(j_userinfo);
-  
-  ulfius_stop_framework(&instance);
-  ulfius_clean_instance(&instance);
-  r_jwk_free(jwk_decrypt);
-}
-END_TEST
-
 START_TEST(test_iddawc_userinfo_response_jwt_nested)
 {
   struct _i_session i_session;
@@ -393,7 +434,7 @@ START_TEST(test_iddawc_userinfo_response_jwt_nested)
                                                     I_OPT_NONE), I_OK);
   ck_assert_int_eq(r_jwks_append_jwk(i_session.client_jwks, jwk_decrypt), RHN_OK);
   ck_assert_int_eq(r_jwks_append_jwk(i_session.server_jwks, jwk_verify), RHN_OK);
-  ck_assert_int_eq(i_load_userinfo(&i_session), I_OK);
+  ck_assert_int_eq(i_get_userinfo(&i_session, 1), I_OK);
   ck_assert_int_eq(json_equal(i_session.j_userinfo, j_userinfo), 1);
   i_clean_session(&i_session);
   json_decref(j_userinfo);
@@ -418,8 +459,9 @@ static Suite *iddawc_suite(void)
   tcase_add_test(tc_core, test_iddawc_userinfo_response_empty);
   tcase_add_test(tc_core, test_iddawc_userinfo_response_char);
   tcase_add_test(tc_core, test_iddawc_userinfo_response_json);
+  tcase_add_test(tc_core, test_iddawc_userinfo_response_json_dpop);
+  tcase_add_test(tc_core, test_iddawc_userinfo_response_json_dpop_invalid);
   tcase_add_test(tc_core, test_iddawc_userinfo_response_jwt_signed);
-  tcase_add_test(tc_core, test_iddawc_userinfo_response_jwt_encrypted);
   tcase_add_test(tc_core, test_iddawc_userinfo_response_jwt_nested);
   tcase_set_timeout(tc_core, 30);
   suite_add_tcase(s, tc_core);
@@ -432,7 +474,7 @@ int main(int argc, char *argv[])
   int number_failed;
   Suite *s;
   SRunner *sr;
-  //y_init_logs("Iddawc", Y_LOG_MODE_CONSOLE, Y_LOG_LEVEL_DEBUG, NULL, "Starting Iddawc oidc openid-configuration loader tests");
+  //y_init_logs("Iddawc", Y_LOG_MODE_CONSOLE, Y_LOG_LEVEL_DEBUG, NULL, "Starting Iddawc oidc userinfo loader tests");
   i_global_init();
   s = iddawc_suite();
   sr = srunner_create(s);
